﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Vintagestory.API.Client;
using Vintagestory.API.Server;
using Vintagestory.API.Config;
using Vintagestory.API.Common;
using Vintagestory.API.Common.Entities;
using Vintagestory.API.MathTools;
using Vintagestory.API.Util;
using Vintagestory.GameContent;
using Vintagestory.API.Datastructures;
using System.Drawing;
using HarmonyLib;

namespace itemizer
{
    public class ItemizerMod : ModSystem
    {
        private readonly Harmony harmony = new Harmony("fr.mobydick.itemizer");

        public ICoreAPI Api;

        public override void Start(ICoreAPI api)
        {
            base.Start(api);

            Api = api;
        }

        public override void StartClientSide(ICoreClientAPI api)
        {
            base.StartClientSide(api);

            var ItemizerCollectibleGetHeldItemInfoOriginal = typeof(CollectibleObject).GetMethod(nameof(CollectibleObject.GetHeldItemInfo));
            var ItemizerCollectibleGetHeldItemInfoPostfix = typeof(Itemizer_CollectibleInfoPatch).GetMethod(nameof(Itemizer_CollectibleInfoPatch.ItemizerCollectibleGetHeldItemInfoPostfix));

            this.harmony.Patch(ItemizerCollectibleGetHeldItemInfoOriginal, postfix: new HarmonyMethod(ItemizerCollectibleGetHeldItemInfoPostfix));

            var ItemizerCollectibleGetNameOriginal = typeof(CollectibleObject).GetMethod(nameof(CollectibleObject.GetHeldItemName));
            var ItemizerCollectibleGetNamePostfix = typeof(Itemizer_CollectibleInfoPatch).GetMethod(nameof(Itemizer_CollectibleInfoPatch.ItemizerCollectibleGetNameInfoPostfix));

            this.harmony.Patch(ItemizerCollectibleGetNameOriginal, postfix: new HarmonyMethod(ItemizerCollectibleGetNamePostfix));
        }

        public override void StartServerSide(ICoreServerAPI api)
        {
            base.StartServerSide(api);

            var parsers = api.ChatCommands.Parsers;
            api.ChatCommands.Create("itemizer")
                .RequiresPrivilege(Privilege.chat)
                .WithArgs(parsers.WordRange("command", new string[] { "name", "desc", "descline", "delname", "deldesc", "lock", "unlock"}))
                .WithArgs(parsers.All("value"))
                .HandleWith(ItemizerCommand);
        }

        private TextCommandResult ItemizerCommand(TextCommandCallingArgs args)
        {
            string lastArgs = "";
            for (int i = 1; i < args.ArgCount; i++)
            {
                lastArgs += args[i].ToString() + " ";
            }

            IPlayer player = args.Caller.Player;

            int spaceIndex = args[0].ToString().IndexOf(" ");

            string opWord = "";
            string value = "";

            if (!args[0].ToString().Contains(" "))
            {
                opWord = args[0].ToString();
            }
            else
            {
                opWord = args[0].ToString().Substring(0, spaceIndex);
                value = args[0].ToString().Substring(spaceIndex + 1, args[0].ToString().Length - opWord.Length - 1);
            }

            if (player.InventoryManager.ActiveHotbarSlot.Empty) return TextCommandResult.Error("[Itemizer] You have to hold an item to use Itemizer.");

            player.InventoryManager.ActiveHotbarSlot.MarkDirty();

            bool notHasArg = value == "" || value == null;

            if (player.InventoryManager.ActiveHotbarSlot.Itemstack.Attributes.GetBool("itemizerLocked", false) && !player.HasPrivilege(Privilege.gamemode))
            {
                string PlayerUid = player.InventoryManager.ActiveHotbarSlot.Itemstack.Attributes.GetString("itemizerLockedBy", "");

                if (PlayerUid != player.PlayerUID)
                {
                    return TextCommandResult.Error("[Itemizer] This item has been locked by " + Api.World.PlayerByUid(player.InventoryManager.ActiveHotbarSlot.Itemstack.Attributes.GetString("itemizerLockedBy", "")) +
                        ". Only them or someone with gamemode privilege can change or unlock this item.");
                }
            }

            switch (opWord)
            {
                case "name":
                    if (notHasArg) return TextCommandResult.Error("[Itemizer] Please provide a name or desc.");
                    player.InventoryManager.ActiveHotbarSlot.Itemstack.Attributes.SetString("itemizerName", value);
                    return TextCommandResult.Success("[Itemizer] Set name to \"" + value + "\"!");
                    break;
                case "desc":
                    if (notHasArg) return TextCommandResult.Error("[Itemizer] Please provide a name or desc.");
                    player.InventoryManager.ActiveHotbarSlot.Itemstack.Attributes.SetString("itemizerDesc", value);
                    return TextCommandResult.Success("[Itemizer] Desc set!");
                    break;
                case "descline":
                    if (notHasArg) return TextCommandResult.Error("[Itemizer] Please provide a name or desc.");
                    player.InventoryManager.ActiveHotbarSlot.Itemstack.Attributes.SetString("itemizerDesc", player.InventoryManager.ActiveHotbarSlot.Itemstack.Attributes.GetString("itemizerDesc", "") + "<br>" + value);
                    return TextCommandResult.Success("[Itemizer] Line added!");
                    break;
                case "delname":
                    player.InventoryManager.ActiveHotbarSlot.Itemstack.Attributes.RemoveAttribute("itemizerName");
                    return TextCommandResult.Success("[Itemizer] Name deleted!");
                    break;
                case "deldesc":
                    player.InventoryManager.ActiveHotbarSlot.Itemstack.Attributes.RemoveAttribute("itemizerDesc");
                    return TextCommandResult.Success("[Itemizer] Desc deleted!");
                    break;
                case "lock":
                    player.InventoryManager.ActiveHotbarSlot.Itemstack.Attributes.SetBool("itemizerLocked", true);
                    player.InventoryManager.ActiveHotbarSlot.Itemstack.Attributes.SetString("itemizerLockedBy", player.PlayerUID);
                    return TextCommandResult.Success("[Itemizer] Item locked! Only you or someone with gamemode privilege can now change its description.");
                    break;
                case "unlock":
                    player.InventoryManager.ActiveHotbarSlot.Itemstack.Attributes.RemoveAttribute("itemizerLocked");
                    return TextCommandResult.Success("[Itemizer] Item unlocked!");
                    break;
                default:
                    return TextCommandResult.Error("[Itemizer] Invalid operaition: \"" + opWord + "\"");
            }
            return TextCommandResult.Success("[Itemizer] Something happened... Please try again");
        }
    }

    public class Itemizer_CollectibleInfoPatch
    {
        [HarmonyPostfix]
        public static void ItemizerCollectibleGetHeldItemInfoPostfix(ItemSlot inSlot, StringBuilder dsc, IWorldAccessor world)
        {
            if (dsc.ToString().Contains("\n\0<i>" + inSlot.Itemstack.Attributes.GetString("itemizerDesc", "") + "</i>\0")) return;
            dsc.Append("\n\0<i>");
            dsc.Append(inSlot.Itemstack.Attributes.GetString("itemizerDesc", ""));
            dsc.Append("</i>\0");
        }

        [HarmonyPostfix]
        public static string ItemizerCollectibleGetNameInfoPostfix(string __result, ItemStack itemStack)
        {
            string name = itemStack.Attributes.GetString("itemizerName", "");

            if (name != "")
            {
                return "<i>" + name + "</i>"; 
            }

            return __result;
        }
    }
}
